package cn.minbb.serialport.impl;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Arrays;

import cn.minbb.serialport.handler.SendDataHandler;
import cn.minbb.serialport.listener.OnDataProgressListener;
import cn.minbb.serialport.utils.LogUtil;

public class SendDataFileImpl implements SendDataHandler<File> {

    @Override
    public void send(FileInputStream fileInputStream, FileOutputStream fileOutputStream, OnDataProgressListener onDataProgressListener, File file) throws IOException {
        FileInputStream fis = new FileInputStream(file);
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        int len;
        byte[] buffer = new byte[64];
        while ((len = fis.read(buffer)) != -1) {
            byteArrayOutputStream.write(buffer, 0, len);
        }
        byte[] byteArray = byteArrayOutputStream.toByteArray();
        fis.close();
        byteArrayOutputStream.close();
        int length = byteArray.length;
        int blockSize = 64, group = length / blockSize + 1;
        int tailSize = length % blockSize;
        for (int index = 1; index <= group; index++) {
            byte[] byteBuffer = new byte[blockSize];
            if (index < group) {
                System.arraycopy(byteArray, (index - 1) * blockSize, byteBuffer, 0, blockSize);
            } else {
                // 最后一组 - 可能是非完整
                byteBuffer = new byte[tailSize];
                System.arraycopy(byteArray, (index - 1) * blockSize, byteBuffer, 0, tailSize);
            }
            if (byteBuffer.length != 0) {
                // 发送数据包
                fileOutputStream.write(byteBuffer);
                LogUtil.setAppendFile("发送文件第" + index + "组，共" + group + "组" + Arrays.toString(byteBuffer));
                if (null != onDataProgressListener) {
                    onDataProgressListener.onProgress(index / (float) group, index, group, byteBuffer);
                }
            }
        }
    }
}
