package cn.minbb.serialport.utils;

import java.nio.ByteBuffer;

public class ByteUtil {

    private static ByteBuffer byteBuffer = ByteBuffer.allocate(8);

    /**
     * 拼接两个 Byte 数组
     *
     * @param data1 Byte 数组 1
     * @param data2 Byte 数组 2
     * @return 拼接后的数组，长度为两个数组长度之和
     */
    public static byte[] addBytes(byte[] data1, byte[] data2) {
        byte[] data3 = new byte[data1.length + data2.length];
        System.arraycopy(data1, 0, data3, 0, data1.length);
        System.arraycopy(data2, 0, data3, data1.length, data2.length);
        return data3;
    }

    public static byte intToByte(int intValue) {
        return (byte) intValue;
    }

    public static int byteToInt(byte byteValue) {
        // Java 总是把 byte 当做有符处理；我们可以通过将其和 0xFF 进行二进制与得到它的无符值
        return byteValue & 0xFF;
    }

    public static int byteArrayToInt(byte[] byteArray) {
        return byteArray[3] & 0xFF | (byteArray[2] & 0xFF) << 8 | (byteArray[1] & 0xFF) << 16 | (byteArray[0] & 0xFF) << 24;
    }

    public static byte[] intToByteArray(int intValue) {
        return new byte[]{(byte) ((intValue >> 24) & 0xFF), (byte) ((intValue >> 16) & 0xFF), (byte) ((intValue >> 8) & 0xFF), (byte) (intValue & 0xFF)};
    }

    public static byte[] longToBytes(long longValue) {
        byte[] b = new byte[8];
        for (int i = 0; i < 8; i++) {
            b[i] = (byte) (longValue >>> (56 - i * 8));
        }
        return b;
    }

    public static byte[] longToByteArray(long longValue) {
        byteBuffer.putLong(0, longValue);
        return byteBuffer.array();
    }

    public static long byteArrayToLong(byte[] byteArray) {
        byteBuffer.put(byteArray, 0, byteArray.length);
        byteBuffer.flip();
        return byteBuffer.getLong();
    }

    /**
     * Hex 字符串转 byte
     *
     * @param inHex Hex 字符串
     * @return byte
     */
    public static byte hexToByte(String inHex) {
        return (byte) Integer.parseInt(inHex, 16);
    }

    /**
     * Hex 字符串转 byte 数组
     *
     * @param inHex Hex 字符串
     * @return byte 数组
     */
    public static byte[] hexToByteArray(String inHex) {
        int hexLength = inHex.length();
        byte[] result;
        if (hexLength % 2 == 1) {
            // 奇数
            hexLength++;
            result = new byte[(hexLength / 2)];
            inHex = "0" + inHex;
        } else {
            // 偶数
            result = new byte[(hexLength / 2)];
        }
        int j = 0;
        for (int i = 0; i < hexLength; i += 2) {
            result[j] = hexToByte(inHex.substring(i, i + 2));
            j++;
        }
        return result;
    }
}
