package cn.minbb.serialport.utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class PackageUtil {

    private static final int PACKAGE_SIZE = 64;

    /**
     * Header： 4 字节。固定为 0x7349AB01。
     * CMD： 1 字节。
     * ARG： 1 字节，表示命令的参数。
     * Reserved： 2 字节。保留字节。
     * Data Length： 4 字节。指出数据区的长度。可以为0。
     * CRC-1： 4 字节，对CMD 到 data Length 部分做 CRC 校验。
     * Data： 数据部分。
     * CRC-2： 4 字节，对Data 部分做 CRC 校验。
     *
     * @param payload 载荷
     * @return 分包列表
     */
    public static List<byte[]> distribute(byte[] payload) {
        List<byte[]> bytesList = new ArrayList<>();
        // 负载长度
        int payloadLength = payload.length;
        // 完整数据长度
        int dataFullLength = PACKAGE_SIZE - 20;
        // 尾部数据长度
        int dataTailLength = payloadLength % dataFullLength;
        // 负载分包总组数
        int group = dataTailLength == 0 ? payloadLength / dataFullLength : payloadLength / dataFullLength + 1;
        byte[] bytesHeader = DataUtil.hexToByteArray("7349AB01");
        byte[] bytesCmd = new byte[1];
        byte[] bytesArg = new byte[1];
        byte[] bytesReserved = new byte[2];
        byte[] bytesDataLength = new byte[4];
        byte[] bytesCrc1 = new byte[4];
        byte[] bytesCrc2 = new byte[4];
        // 遍历有效负载，分包，添加包头等数据
        for (int index = 1; index <= group; index++) {
            byte[] bytes = new byte[0];
            byte[] bytesData = new byte[dataFullLength];
            if (index < group) {
                bytesDataLength = ByteUtil.intToByteArray(dataFullLength);
                System.arraycopy(payload, (index - 1) * dataFullLength, bytesData, 0, dataFullLength);
            } else {
                // 最后一组 - 可能是非完整
                bytesDataLength = ByteUtil.intToByteArray(dataTailLength);
                bytesData = new byte[dataTailLength];
                System.arraycopy(payload, (index - 1) * dataFullLength, bytesData, 0, dataTailLength);
            }
            // 组装头部
            bytesCmd[0] = 1;
            bytesArg[0] = 1;
            bytesReserved[0] = 1;
            bytesReserved[1] = 1;
            bytes = ByteUtil.addBytes(bytes, bytesHeader);
            bytes = ByteUtil.addBytes(bytes, bytesCmd);
            bytes = ByteUtil.addBytes(bytes, bytesArg);
            bytes = ByteUtil.addBytes(bytes, bytesReserved);
            bytes = ByteUtil.addBytes(bytes, bytesDataLength);
            // 校验数据
            bytesCrc1 = ByteUtil.intToByteArray(CRCUtil.getCrc16(bytes, 4, 8));
            bytesCrc2 = ByteUtil.intToByteArray(CRCUtil.getCrc16(bytesData));
            // 组装校验结果和数据部分
            bytes = ByteUtil.addBytes(bytes, bytesCrc1);
            bytes = ByteUtil.addBytes(bytes, bytesData);
            bytes = ByteUtil.addBytes(bytes, bytesCrc2);
            bytesList.add(bytes);
            // LogUtil.setAppendFile("组装报文" + index + "组，共" + group + "组" + Arrays.toString(bytesData));
        }
        return bytesList;
    }

    /**
     * 数据包解析
     *
     * @param bytesList 数据包列表
     * @param serialId  流水号
     */
    public static String analyze(List<byte[]> bytesList, String serialId) {
        byte[] bytesBuffer = new byte[0];
        for (byte[] bytes : bytesList) {
            bytesBuffer = ByteUtil.addBytes(bytesBuffer, bytes);
            LogUtil.setAppendFile("serialId:" + Arrays.toString(bytes));
        }
        LogUtil.setAppendFile("serialId:" + new String(bytesBuffer));
        return serialId + ":" + new String(bytesBuffer);
    }
}
